package halmaklient;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.StringTokenizer;

import javax.swing.JLabel;
import javax.swing.JPanel;

public class HalmaLaud extends JPanel implements ActionListener {
	
	public static int TYHI = 0;
	public static int MUST = 1;
	public static int VALGE = -1;
	
	private static final long serialVersionUID = -8968625492394443225L;

	int[][] laud = new int[10][10];
	HamaRuut[][] ruudud = new HamaRuut[10][10];
	
	public HalmaLaud() {
		super(new GridLayout(10, 10));

		this.teeRuudud();
		this.initLaud();
	}

	private void teeRuudud() {
		// lemine rida
		this.add(new JLabel());
		for (int i = 1; i <= 8; i++)
			this.add(new JLabel("" + i, JLabel.CENTER));
		this.add(new JLabel());
		
		// ruudustik
		for (int j = 8; j >= 1; j--) {
			this.add(new JLabel("" + j, JLabel.CENTER));
			for (int i = 1; i <= 8; i++) {
				HamaRuut hr = new HamaRuut(i, j);
				hr.addActionListener(this);
				ruudud[i][j] = hr;
				this.add(hr);
			}
			this.add(new JLabel("" + j, JLabel.CENTER));
		}
		
		// alumine rida
		this.add(new JLabel());
		for (int i = 1; i <= 8; i++)
			this.add(new JLabel("" + i, JLabel.CENTER));
		this.add(new JLabel());
		
		this.setPreferredSize(new Dimension(400, 400));
	}
	
	public void initLaud() {
		this.tyhistaSeisundid();
		for (int i = 1; i <= 8; i++)
			for (int j = 1; j <= 8; j++) {
				if (i <= 4 && j <= 3) this.seaSeis(i, j, MUST);
				else if (i >= 5 && j >= 6) this.seaSeis(i, j, VALGE);
				else this.seaSeis(i, j, TYHI);
			}
	}
	
	public void tyhistaSeisundid() {
		for (int i = 1; i <= 8; i++)
			for (int j = 1; j <= 8; j++) 
				this.ruudud[i][j].tyhistaSeisundid();
	}
	
	private void seaSeis(int v, int r, int s) {
		this.laud[v][r] = s;
		this.ruudud[v][r].seaSeis(s);
	}

	public boolean mangLabi() {
		for (int i = 1; i <= 4; i++)
			for (int j = 1; j <= 3; j++)
				if (this.laud[i][j] != VALGE) return false;
		
		for (int i = 5; i <= 8; i++)
			for (int j = 6; j <= 8; j++)
				if (this.laud[i][j] != MUST) return false;
		
		return true;
	}

	private boolean annaKaik = false;
	private Object kaik = new Object();
	private int stepCnt = 0;
	private int varv = 0;
	private int v0;
	private int r0;
	private int v1;
	private int r1;
	
	public String annaKaik(int varv) {
		synchronized (this.kaik) {
			this.stepCnt = 0;
			this.varv = varv;
			this.annaKaik = true;
			try {
				this.kaik.wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			this.annaKaik = false;

			int s = this.laud[this.v0][this.r0];
			this.seaSeis(this.v0, this.r0, TYHI);
			this.seaSeis(this.v1, this.r1, s);
			
			return this.v0 + " " + this.r0 + " " + this.v1 + " " + this.r1;
		}
	}

	public void teeKaik(String loe) {
		try {
			StringTokenizer st = new StringTokenizer(loe);
			int v0 = Integer.parseInt(st.nextToken(), 10);
			int r0 = Integer.parseInt(st.nextToken(), 10);
			int v1 = Integer.parseInt(st.nextToken(), 10);
			int r1 = Integer.parseInt(st.nextToken(), 10);
			
			int s = this.laud[v0][r0];
			this.seaSeis(v0, r0, TYHI);
			this.ruudud[v0][r0].seaValitud(true);
			this.seaSeis(v1, r1, s);
			this.ruudud[v1][r1].seaVoib(true);
		} catch (Exception e) {
			throw new IllegalStateException("Sisend serverilt: \"" + loe + "\"\nViga: " + e.getMessage(), e);
		}
	}

	public void actionPerformed(ActionEvent e) {
		synchronized (this.kaik) {
			if (!this.annaKaik) return;
			StringTokenizer st = new StringTokenizer(e.getActionCommand(), "[,]");
			int v = Integer.parseInt(st.nextToken(), 10);
			int r = Integer.parseInt(st.nextToken(), 10);
			
			if (this.stepCnt == 0) {
				this.valiNupp(v, r);
			} else {
				if (saabKaia(v, r)) {
					this.v1 = v;
					this.r1 = r;
					this.ruudud[v][r].seaVoib(true);
					this.stepCnt++;
					this.kaik.notifyAll();
				} else {
					this.valiNupp(v, r);
				}
			}
		}
	}
	
	private void valiNupp(int v, int r) {
		if (this.laud[v][r] != this.varv) return;
		
		this.v0 = v;
		this.r0 = r;
		this.tyhistaSeisundid();
		this.ruudud[v][r].seaValitud(true);
		this.stepCnt = 1;
		
		for (int i = 1; i <= 8; i++)
			for (int j = 1; j <= 8; j++)
				if (this.saabKaia(i, j))
					this.ruudud[i][j].seaVoib(true);
	}
	
	private boolean saabKaia(int v, int r) {
		if (this.laud[v][r] != TYHI) return false;
		if (this.v0 != v && this.r0 != r) return false;
		
		int dv = v - this.v0;
		int dr = r - this.r0;

		// samm
		if (Math.abs(dv) == 1 || Math.abs(dr) == 1) return true;
		
		// hpe
		if (dv != 0 && Math.abs(dv) % 2 == 1) return false;
		if (dr != 0 && Math.abs(dr) % 2 == 1) return false;
		
		if (this.laud[(this.v0 + v) / 2][(this.r0 + r) / 2] == TYHI) return false;
		
		return true;
	}

}
